"""Data fetching module."""
from __future__ import annotations

import requests

from tcmb import auth

URL = "https://evds2.tcmb.gov.tr/service/evds"


def _create_uri(params: dict, endpoint: str | None = None) -> str:
    """Create URI using the params and the evds url.

    The official api does not use `?` for query strings.
    Therefore the parameters are added to the end of the url
    instead of passing to the requests.get() function as `params`.

    Parameters
    ----------
    params:
        Query parameters.
    endpoint:
        One of
        - "categories"
        - "serieList"
        - "datagroups"
        - None
    """
    if endpoint is not None:
        endpoint = endpoint + "/"
    else:
        endpoint = ""

    # create query string
    # NOTE: alternative -> urllib.parse.urlencode(params)
    #       however, this keeps None values
    query_str = "&".join([k + "=" + str(v) for k, v in params.items() if v is not None])

    return f"{URL}/{endpoint}{query_str}"


def get_response(
    params: dict,
    headers: dict | None = None,
    endpoint: str | None = None,
    proxies: dict | None = None,
    **kwargs,
):
    """Get response.

    Parameters
    ----------
    params:
        Request parameters.
    headers:
        Request headers.
    endpoint:
        TCMB Web Service API endpoint.
        One of
        - "categories"
        - "serieList"
        - "datagroups"
        - None
    proxies:
        Dictionary mapping protocol to the URL of the proxy.
    kwargs:
        Optional keyword arguments that request takes.
    """
    # Create url for the get request
    #  NOTE: the official api does not use ? for query strings
    #  Therefore the parameters are added to the end of the url
    #  instead of passing to the get request as params
    url = _create_uri(params=params, endpoint=endpoint)

    # Get request
    res = requests.get(url, headers=headers, proxies=proxies, **kwargs)

    # Check status
    auth.check_status(res)

    return res



import json
import os.path


def read_package_data() -> list:
    """Read series codes from the package resources."""
    file_path = os.path.join(os.path.dirname(__file__), "resources", "series.json")

    with open(file_path, "r") as file:
        dg_series = json.load(file)

    series_list = []

    for item in dg_series.values():
        series_list.extend(item)

    return series_list


def fetch_dg_series_codes() -> dict:
    """Get series codes for each datagroup."""
    from tcmb import Client

    client = Client()
    dg_codes = [dg["DATAGROUP_CODE"] for dg in client.datagroups]

    # There are 439 data groups as of 2023-04-25
    print(f"There are {len(dg_codes)} items in total.")

    dg_series = {}
    for dg_code in dg_codes:
        series_codes = []
        series_metadata = client.get_series_metadata(datagroup=dg_code)

        if isinstance(series_metadata, dict):
            series_metadata = [series_metadata]

        for item in series_metadata:
            series_codes.append(item["SERIE_CODE"])

        dg_series[dg_code] = series_codes

    return dg_series


def _update_package_data():
    """Update package resources.

    This function is to be used by the maintainer to syncronize
    package data with the current data available on EVDS Web Service.
    """
    dg_series = fetch_dg_series_codes()

    file_path = os.path.join(os.path.dirname(__file__), "resources", "series.json")

    with open(file_path, "w") as file:
        json.dump(dg_series, file)



"""Core module.

This module allows user to read data from
Turkish Central Bank's (TCMB) EVDS Web Service.

See for details:
----------------
EN: https://evds2.tcmb.gov.tr/help/videos/EVDS_Web_Service_Usage_Guide.pdf
TR: https://evds2.tcmb.gov.tr/help/videos/EVDS_Web_Servis_Kullanim_Kilavuzu.pdf

"""
from __future__ import annotations
from datetime import date
from functools import cached_property
import os

import pandas as pd

from tcmb import const, auth, utils, fetch


def read(
    series: str | list[str],
    start: str | None = None,
    end: str | None = None,
    agg: str | list[str] | None = None,
    formulas: str | None = None,
    freq: str | int | None = None,
    seperator: str = ".",
    headers: dict | None = None,
    api_key: str | None = None,
    **kwargs,
) -> pd.DataFrame:
    """Read data from TCMB's EVDS Web Service.

    Parameters
    ----------
    series:
        Time series key. If str, wildcard characters (*) and (?) can be used.
        The wildcard characters are represented as an asterisk (*)
        or a question mark (?). The asterisk (*) represents any number of characters,
        while the question mark (?) represents a single character.
        Additionally, omitting the value has the same effect as using an asterisk.
    start: str or datetime-like
        Start date of the series in DD-MM-YYYY, YYYY-MM-DD, or datetime-like
        format.
    end: str or datetime-like
        End date of the series in DD-MM-YYYY, YYYY-MM-DD, or datetime-like
        format.
    agg: {None, "avg", "min", "max", "first", "last", "sum"}
        Aggregation type of the result. If type of the `series` is string,
        type of the `agg` must also be string. If `series` is a list,
        `agg` can be either a `str` or a `list`.
    formulas:
        Compute values such as percentage change, difference,
        year-over-year (YoY) chanhe, Year-to-date (YTD) change,
        moving average.
        - 0: level
        - 1: percentage change
        - 2: difference
        - 3: YoY percentage change
        - 4: YoY difference
        - 5: YTD percentage change
        - 6: YTD difference
        - 7: moving average
        - 8: moving sum
    freq:
        Frequency of the series. Either frequency string or int to be used.
        - None: the default frequency of the series is taken.
        - "D": 1,      # daily
        - "B": 2,      # business
        - "W-FRI": 3,  # weekly (friday)
        - "W": 3,      # weekly (friday)
        - "M": 5,      # monthly
        - "Q": 6,      # quarterly
        - "2Q": 7,     # semi-annual
        - "A": 8,      # annual
        - "Y": 8,      # annual
    seperator:
        Delimiter to use.
    api_key:
        API key of the user. If None, environment variable with
        the name "TCMB_API_KEY" will be used to as the api_key. If
        the api_key argument is not passed or it is not exported to the
        environment, ApiKeyError is raised. EVDS Web Service accepts API Key
        as a request header in the format {"key": api_key}.
    **kwargs:
        Keyword arguments passed to the request.

    Returns
    -------
    df: Pandas DataFrame.

    References
    ----------
    - https://evds2.tcmb.gov.tr/help/videos/EVDS_Web_Service_Usage_Guide.pdf

    Example
    -------
    import tcmb

    # read one series
    df = tcmb.read("TP.DK.USD.S.YTL")

    # read multiple series
    df = tcmb.read(["TP.DK.USD.S.YTL", "TP.DK.EUR.S.YTL"])

    # read series with wildcarding
    df = tcmb.read("TP.DK.*.S.YTL"])
    """
    api_key = api_key or os.environ.get("TCMB_API_KEY")

    # convert freq string to
    if isinstance(freq, str):
        freq = const.FREQ_MAPPING[freq]

    # wildcard search if any of the following case
    if isinstance(series, str):
        if ("*" in series) or ("?" in series) or (".." in series):
            series = utils.wildcard_search(series)

    # convert list to str seperated by "-"
    # as descibed in the api reference
    if isinstance(series, list):
        series_str = "-".join(series)
    else:
        series_str = series

    # convert list to str seperated by "-"
    if isinstance(agg, list):
        agg = "-".join(agg)

    # convert date string formats to "DD-MM-YYYY"
    if start:
        start = utils.standardize_date(start)
    if end:
        end = utils.standardize_date(end)

    params = {
        "series": series_str,
        "startDate": start or "01-01-1970",
        "endDate": end or date.today().strftime("%d-%m-%Y"),
        "type": "json",  # csv, xml, json
        "aggregationTypes": agg,
        "formulas": formulas,
        "frequency": freq,
        "decimalSeperator": seperator,
        **kwargs,
    }

    # add api_key to request header
    if headers is None:
        headers = {"key": api_key}
    elif "key" not in headers:
        headers["key"] = api_key

    res = fetch.get_response(params=params, headers=headers)

    # convert response JSON to DataFrame
    #   time series data is in the "items"
    data = utils.to_dataframe(res.json()["items"], series=series)

    return data


class Client:
    """Base class for TCMB web service access.

    Parameters
    ----------
    api_key:
        API key of the user. If None, environment variable with
        the name "TCMB_API_KEY" will be used to as the api_key. If
        the api_key argument is not passed or it is not exported to the
        environment, ApiKeyError is raised.
    """

    def __init__(self, api_key: str | None = None):
        self.api_key = api_key or os.environ.get("TCMB_API_KEY")
        auth.check_api_key(self.api_key)

    @staticmethod
    def _get_response(
        params: dict,
        headers: dict | None = None,
        endpoint: str | None = None,
        proxies: dict | None = None,
        **kwargs,
    ):
        """Get response.

        Parameters
        ----------
        params:
            Request parameters.
        headers:
            Request headers.
        endpoint:
            TCMB Web Service API endpoint.
            One of
            - "categories"
            - "serieList"
            - "datagroups"
            - None
        proxies:
            Dictionary mapping protocol to the URL of the proxy.
        kwargs:
            Optional keyword arguments that request takes.
        """
        res = fetch.get_response(
            params=params,
            headers=headers,
            endpoint=endpoint,
            proxies=proxies,
            **kwargs,
        )

        return res

    def read(
        self,
        series: str | list[str],
        start: str | None = None,
        end: str | None = None,
        agg: str | list[str] | None = None,
        formulas: str | None = None,
        freq: str | int | None = None,
        seperator: str = ".",
        headers: dict | None = None,
        metadata: bool = False,
        **kwargs,
    ) -> pd.DataFrame | dict:
        """Read data from TCMB's EVDS Web Service.

        Parameters
        ----------
        series:
            Time series key. e.g. "TP.DK.USD.S.YTL"
        start: str or datetime-like
            Start date of the series in DD-MM-YYYY, YYYY-MM-DD, or datetime-like
            format.
        end: str or datetime-like
            End date of the series in DD-MM-YYYY, YYYY-MM-DD, or datetime-like
            format.
        agg: {None, "avg", "min", "max", "first", "last", "sum"}
            Aggregation type of the result. If type of the `series` is string,
            type of the `agg` must also be string. If `series` is a list,
            `agg` can be either a `str` or a `list`.
        formulas:
            Compute values such as percentage change, difference,
            year-over-year (YoY) chanhe, Year-to-date (YTD) change,
            moving average.
            - 0: level
            - 1: percentage change
            - 2: difference
            - 3: YoY percentage change
            - 4: YoY difference
            - 5: YTD percentage change
            - 6: YTD difference
            - 7: moving average
            - 8: moving sum
        freq:
            Frequency of the series. Either frequency string or int to be used.
            - None: the default frequency of the series is taken.
            - "D": 1,      # daily
            - "B": 2,      # business
            - "W-FRI": 3,  # weekly (friday)
            - "W": 3,      # weekly (friday)
            - "M": 5,      # monthly
            - "Q": 6,      # quarterly
            - "2Q": 7,     # semi-annual
            - "A": 8,      # annual
            - "Y": 8,      # annual
        seperator:
            Delimiter to use.
        metadata:
            Whether to read metadata for the series. If True, get requests are
            performed as many as the number of series keys passed. The metadata
            can be accessed using the `.attrs` attribute of the pandas.DataFrame.
            e.g. df.attrs
        **kwargs:
            Keyword arguments passed to the request.

        Returns
        -------
        df: Pandas DataFrame.

        References
        ----------
        - https://evds2.tcmb.gov.tr/help/videos/EVDS_Web_Service_Usage_Guide.pdf

        Example
        -------
        df = client.read(["...", "..."])
        """
        df = read(
            series=series,
            start=start,
            end=end,
            agg=agg,
            formulas=formulas,
            freq=freq,
            seperator=seperator,
            headers=headers,
            api_key=self.api_key,
            **kwargs,
        )

        # if metadata is true, save metadata info
        # to the .attrs attribute of the DataFrame
        #   it can be accessed using attrs: e.g. "df.attrs"
        if metadata:
            attrs: dict[str, dict] = {}
            # TODO: update for the case of list of series
            for series_item in series.split("-"):
                attrs[series_item] = self.get_series_metadata(series_item)

            df.attrs = attrs

        return df

    def get_categories_metadata(self):
        """Get the list of the metadata of all categories."""
        params = {"type": "json"}

        # add api_key to request header
        headers = {"key": self.api_key}

        res = self._get_response(params=params, endpoint="categories", headers=headers)

        return res.json()

    def get_datagroups_metadata(
        self, mode: int = 0, code: str | int | None = None
    ) -> pd.DataFrame:
        """Get datagroups metadata.

        The metadata of the datagroups include the following:
        ['DATAGROUP_CODE', 'CATEGORY_ID', 'DATAGROUP_NAME', 'DATAGROUP_NAME_ENG',
         'FREQUENCY_STR', 'FREQUENCY', 'DATASOURCE', 'DATASOURCE_ENG',
         'METADATA_LINK', 'METADATA_LINK_ENG', 'REV_POL_LINK',
         'REV_POL_LINK_ENG', 'APP_CHA_LINK', 'APP_CHA_LINK_ENG', 'START_DATE',
         'END_DATE']

        Parameters
        ----------
        mode: {0, 1, 2}
            Apply filtering according to the following options.
            - 0: Return all data groups of all categories
            - 1: Return information of the selected data group
            - 2: Return information of all data groups of a category
            If 1 or 2 is passed, "code" argument must also be passed.
        code:
            - If mode=1, the user must pass a "datagroup code"
              e.g. datagroups(mode=1, code="bie_yssk")
            - If mode=2, the user must pass a "category code" (category ID).
              e.g. datagroups(mode=2, code=2)

        """
        if (mode > 0) and (code is None):
            raise ValueError("If mode=1 or mode=2, 'code' argument must be passed.")

        params = {
            "mode": str(mode),
            "code": code,
            "type": "json",
        }

        # add api_key to request header
        headers = {"key": self.api_key}

        res = self._get_response(params=params, endpoint="datagroups", headers=headers)

        json_data = res.json()

        # if no data (empty response)
        if not json_data:
            raise ValueError(
                "No data on respose, check `mode` and `code` parameters.\n"
                f"Response: {res.content}"
            )

        return json_data

    def get_series_metadata(
        self, series: str | None = None, datagroup: str | None = None
    ) -> dict | list[dict]:
        """Get the metadata of the series.

        The metadata of the series include the following:
        ['SERIE_CODE', 'DATAGROUP_CODE', 'SERIE_NAME', 'SERIE_NAME_ENG',
         'FREQUENCY_STR', 'DEFAULT_AGG_METHOD_STR', 'DEFAULT_AGG_METHOD',
         'TAG', 'TAG_ENG', 'DATASOURCE', 'DATASOURCE_ENG', 'METADATA_LINK',
         'METADATA_LINK_ENG', 'REV_POL_LINK', 'REV_POL_LINK_ENG',
         'APP_CHA_LINK', 'APP_CHA_LINK_ENG', 'START_DATE', 'END_DATE']

        Parameters
        ----------
        series:
            Time series key. e.g. "TP.DK.USD.S.YTL"
            Onde of `series` or `code` must be passed.
        datagroup:
            Data group code. eg. "bie_yssk".
            One of `series` or `code` must be passed.

        Returns
        -------
        One of
        - Dict of metadata of one time series
        - List of multiple metadata of multiple time series
        """
        if (series is None) and (datagroup is None):
            raise ValueError("One of `series` or `code` arguments must be passed.")

        params = {
            "code": series or datagroup,
            "type": "json",
        }

        # add api_key to request header
        headers = {"key": self.api_key}

        res = self._get_response(params=params, endpoint="serieList", headers=headers)

        return res.json()

    @cached_property
    def categories(self):
        """Get categories metadata and save as a property."""
        return self.get_categories_metadata()

    @cached_property
    def datagroups(self):
        """Get datagroups metadata and save as a property."""
        return self.get_datagroups_metadata()

    def __repr__(self) -> str:
        return "tcmb client"



from tcmb import utils

import pytest


test_date_data = [
    ("01-01-2024", "01-01-2024"),
    ("17.12.2009", "17-12-2009"),
    ("2011-06-18", "18-06-2011"),
    ("2018.04.10", "10-04-2018"),
]


@pytest.mark.parametrize("date_str, expected", test_date_data)
def test_standardize_date(date_str, expected):
    result = utils.standardize_date(date_str)
    print(result)
    assert result == expected


def test_wildcard_search():
    items_expected = [
        "TP.API.REP.TL.A12",
        "TP.API.REP.TL.A23",
        "TP.API.REP.TL.G1",
        "TP.API.REP.TL.G1530",
        "TP.API.REP.TL.G214",
    ]
    items = utils.wildcard_search("TP.API.REP.TL.*")

    assert all(item in items_expected for item in items)



"""Errors module."""


class ApiKeyError(ValueError):
    """Raise for API Key error."""

    def __init__(self, msg="Check API key!", *args, **kwargs):
        super().__init__(msg, *args, **kwargs)


class InvalidSeriesCode(ValueError):
    """Raise for Invalid Series Code error."""

    def __init__(
        self,
        msg=(
            "Invalid series code! "
            "One of the series codes is not valid, "
            "or there are too many codes."
        ),
        *args,
        **kwargs
    ):
        super().__init__(msg, *args, **kwargs)



"""Authentication module."""
from __future__ import annotations
import re

import requests
from requests.exceptions import HTTPError, JSONDecodeError

from tcmb.errors import ApiKeyError, InvalidSeriesCode


def _extract_error_message(response):
    """Extract error message from the html body"""
    # Extract error message from the html body
    pattern = r"<body>(.*?)</body>"
    error_msgs = re.findall(pattern, str(response.content))
    error_msg = error_msgs[0].strip("\\rn")

    return error_msg


def check_status(response):
    """Check if response is ok and authentication is done."""
    if response.status_code != 200:
        try:
            response.raise_for_status()
        except HTTPError as err:
            # get error message
            error_msg = _extract_error_message(response)
            print(f"Response Error Message: {error_msg}")

            # evds responds with an 500 Internal Server Error for almost
            # any issues. This error response is a generic "catch-all" response.
            # Therefore the traceback is printed as well as the most
            # probable cause of the HTTPError: ApiKeyError
            print(err.with_traceback(None))
            raise ApiKeyError(
                "API key is invalid or wrong key. See error message for details."
            ) from err
    else:
        try:
            response.json()
        except JSONDecodeError:
            if "error-title" in response.text:
                raise InvalidSeriesCode()
            else:
                print("Cannot convert response to JSON")
                raise


def check_api_key(api_key: str | None = None) -> bool:
    """Check API key.

    TCMB Web Service does not provide a way to check the api key.
    This function allows checking api key with the "categories"
    endpoint which doesn't reques a series key parameter.

    Parameters
    ----------
    api_key:
    """
    if api_key is None:
        raise ApiKeyError("No API key provided.")

    headers = {"key": api_key}

    res = requests.get(
        "https://evds2.tcmb.gov.tr/service/evds/categories/type=json",
        headers=headers,
        timeout=30,
    )
    # check if authenticated
    check_status(res)

    return True



import pytest
import requests

from tcmb import Client


# custom class to be the mock return value
# will override the requests.Response returned from requests.get
class MockResponse:
    status_code = 200

    @staticmethod
    def json():
        """mock json() method returns a testing dictionary."""
        return {"mock_key": "mock_response"}

    @staticmethod
    def raise_for_status():
        return None


@pytest.fixture
def mock_response(monkeypatch):
    """Requests.get() mocked to return {'mock_key':'mock_response'}."""

    def mock_get(*args, **kwargs):
        return MockResponse()

    monkeypatch.setattr(requests, "get", mock_get)


@pytest.fixture(autouse=True)
def no_requests(monkeypatch):
    """Remove requests.sessions.Session.request for all tests."""
    monkeypatch.delattr("requests.sessions.Session.request")


def test_client(mock_response):
    client = Client(api_key="fakekey")


def test_get_response(mock_response):
    client = Client(api_key="fakekey")
    result = client._get_response(params={"mock_param_key": "mock_param_value"})
    assert result.json()["mock_key"] == "mock_response"


def test_categories(mock_response):
    client = Client(api_key="fakekey")
    result = client.categories
    assert result["mock_key"] == "mock_response"


def test_datagroups(mock_response):
    client = Client(api_key="fakekey")
    result = client.datagroups
    assert result["mock_key"] == "mock_response"



"""Utilities module."""
from __future__ import annotations

import json
import re
import os.path
from datetime import datetime


import numpy as np
import pandas as pd

from tcmb._data import fetch_dg_series_codes


def standardize_date(date_str: str) -> str:
    """Standardize date string format to output DD-MM-YYYY.

    TCMB Web Service accepts dates in DD-MM-YYYY format only.
    This function converts datetime dtype and date string
    in YYYY-MM-DD format into the default TCMB format.

    Paramters
    ---------
    date_str:
        Date string in one of the following formats:
        - DD-MM-YYYY
        - DD.MM.YYYY
        - YYYY-MM-DD
        - YYYY.MM.DD

    Returns
    -------
    date_str:
        Date string in the "DD-MM-YYYY" format.
    """

    if re.match(r"\d{1,2}-\d{1,2}-\d{4}", date_str):
        date_format = "%d-%m-%Y"
    elif re.match(r"\d{1,2}.\d{1,2}.\d{4}", date_str):
        date_format = "%d.%m.%Y"
    elif re.match(r"\d{4}-\d{1,2}-\d{1,2}", date_str):
        date_format = "%Y-%m-%d"
    elif re.match(r"\d{4}.\d{1,2}.\d{1,2}", date_str):
        date_format = "%Y.%m.%d"

    return datetime.strptime(date_str, date_format).strftime("%d-%m-%Y")


def to_dataframe(data: dict, series: str | list) -> pd.DataFrame:
    """Convert data from the json response to pandas DataFrame."""
    if isinstance(series, str):
        series = [series]

    # generate column names from series keys by replacing "." with "_"
    # these columns will be selected as the value columns
    # the columns other than date (Tarih) and value columns will be dropped
    value_cols = [col.replace(".", "_") for col in series]

    df = pd.DataFrame(data)
    # drop unused column
    df = df[["Tarih", *value_cols]]
    # set date as index
    # TODO: check if "Tarih" always the date column
    df = df.set_index("Tarih")

    # detect date format
    if re.match(r"\d+-\d+-\d{4}", df.index[0]):
        date_format = "%d-%m-%Y"
    elif re.match(r"\d+-\d{4}", df.index[0]):
        date_format = "%m-%Y"
    elif re.match(r"\d{4}-\d+", df.index[0]):
        date_format = "%Y-%m"
    elif re.match(r"\d{4}", df.index[0]):
        date_format = "%Y"

    # convert date strings to datetime
    df.index = pd.to_datetime(df.index, format=date_format)

    # replace None with NaN
    df = df.fillna(np.nan)
    # convert object columns to float
    # TODO: convert to integer when possible
    df = df.astype(float)
    # drop rows if all missing
    df = df.dropna(how="all")

    return df


def wildcard_search(
    pattern: str, items: list | None = None, use_package_data: bool = True
) -> list:
    """Search for items using regex pattern that can contain wildcard characters.

    Parameters
    ----------
    pattern:
        The regex pattern to use for searching, which may contain wildcard
        characters. The wildcard characters are represented as an asterisk (*)
        or a question mark (?). The asterisk (*) represents any number of characters,
        while the question mark (?) represents a single character.
        Additionally, omitting the value has the same effect as using an asterisk.
    items:
        Use the list of items to search through. If None, items in the
        package data is used. Note that package data may not be up to date.
        The user can choose to fetch series data from TCMB instead of
        using the flat file in package data by using the "update" parameter.
    use_package_data:
        Whether to use package resources or fetch all series keys from
        the TCMB database. If False, fetching may take up to 5 minutes.

    Returns
    -------
    A list of items that match the regex pattern.

    Example
    -------
    >>> wildcard_search('TP.API.REP.TL.*', items=items)
    ['TP.API.REP.TL.A12',
     'TP.API.REP.TL.A23',
     'TP.API.REP.TL.G1',
     'TP.API.REP.TL.G1530',
     'TP.API.REP.TL.G214']

    >>> wildcard_search('TP.API.REP.TL.A??', items=items)
    ['TP.API.REP.TL.A12', 'TP.API.REP.TL.A23']

    """
    if items is None:
        items = []

        if use_package_data:
            file_path = os.path.join(
                os.path.dirname(__file__), "resources", "series.json"
            )

            with open(file_path, "r") as file:
                dg_series = json.load(file)

        else:
            dg_series = fetch_dg_series_codes()

        # merge series of all datagroups into one list
        for item in dg_series.values():
            items.extend(item)

    # Replace the wildcard characters with a regex-friendly equivalent.
    pattern = pattern.replace("*", ".*").replace("?", ".")

    # Compile the regex pattern for efficiency.
    compiled_regex = re.compile(pattern)

    # Use list comprehension to find matching items.
    matching_items = [item for item in items if compiled_regex.search(item)]

    return matching_items



# freq_str : freq_id mapping
FREQ_MAPPING = {
    "D": 1,  # daily
    "B": 2,  # business daily
    "W-FRI": 3,  # weekly (friday)
    "W": 3,  # weekly (friday)
    "M": 5,  # monthly
    "Q": 6,  # quarterly
    "2Q": 7,  # semi-annual
    "A": 8,  # annual
    "Y": 8,  # annual
}

FREQUENCY_TR_MAPPING = {
    "AYLIK": "M",
}
